@@ -45,35 +45,75 @@ int display_mode;
 int Interactive = 1;
 int PrintVersion = 0;
 int PrintHelp = 0;
-int MaxPing = 16;
+int MaxPing = 10;
+int ForceMaxPing = 0;
 float WaitTime = 1.0;
 char *Hostname = NULL;
 char *InterfaceAddress = NULL;
 char LocalHostname[128];
 int dns = 1;
-int packetsize = MINPACKET;
+int packetsize = 64;		/* default packet size */
+int bitpattern = 0;
+int tos = 0;
+/* begin ttl windows addByMin */
+int fstTTL = 1;			/* default start at first hop */
+//int maxTTL = MaxHost-1;		/* max you can go is 255 hops */
+int maxTTL = 30;		/* inline with traceroute */
+/* end ttl */
 
-void parse_arg(int argc, char **argv) {
+#ifdef ENABLE_IPV6
+#define DEFAULT_AF AF_UNSPEC
+#else
+#define DEFAULT_AF AF_INET
+#endif
+
+
+#ifdef NO_HERROR
+#define herror(str) printf(str ": error looking up \"%s\"\n", Hostname);
+#endif
+
+int af = DEFAULT_AF;
+
+/* default display field(defined by key in net.h) and order */
+char fld_active[2*MAXFLD] = "LS NABWV";
+
+
+
+void parse_arg(int argc, char **argv) 
+{
   int opt;
   static struct option long_options[] = {
     { "version", 0, 0, 'v' },
     { "help", 0, 0, 'h' },
+
     { "report", 0, 0, 'r' },
-    { "report-cycles", 1, 0, 'c' },
+    { "xml", 0, 0, 'x' },
     { "curses", 0, 0, 't' },
     { "gtk", 0, 0, 'g' },
-    { "interval", 1, 0, 'i' },
-    { "psize", 1, 0, 'p' },
-    { "no-dns", 0, 0, 'n' },
-    { "split", 0, 0, 's' },     /* BL */
-    { "address", 1, 0, 'a' },
     { "raw", 0, 0, 'l' },
+    { "split", 0, 0, 'p' },     /* BL */
+    				/* maybe above should change to -d 'x' */
+
+    { "order", 1, 0, 'o' },	/* fileds to display & their order */
+
+    { "interval", 1, 0, 'i' },
+    { "report-cycles", 1, 0, 'c' },
+    { "psize", 1, 0, 's' },	/* changed 'p' to 's' to match ping option
+				   overload psize<0, ->rand(min,max) */
+    { "bitpattern", 1, 0, 'b' },/* overload b>255, ->rand(0,255) */
+    { "tos", 1, 0, 'Q' },	/* typeof service (0,255) */
+    { "no-dns", 0, 0, 'n' },
+    { "address", 1, 0, 'a' },
+    { "first-ttl", 1, 0, 'f' },	/* -f & -m are borrowed from traceroute */
+    { "max-ttl", 1, 0, 'm' },
     { 0, 0, 0, 0 }
   };
 
   opt = 0;
   while(1) {
-    opt = getopt_long(argc, argv, "a:hvrc:tgklnsi:p:", long_options, NULL);
+    /* added f:m:o: byMin */
+    opt = getopt_long(argc, argv,
+		      "vhrxtglpo:i:c:s:b:Q:na:f:m:", long_options, NULL);
     if(opt == -1)
       break;
 
@@ -84,30 +124,40 @@ void parse_arg(int argc, char **argv) {
     case 'h':
       PrintHelp = 1;
       break;
+
     case 'r':
       DisplayMode = DisplayReport;
       break;
-    case 'c':
-      MaxPing = atoi (optarg);
-      break;
-    case 'p':
-      packetsize = atoi (optarg);
-      break;
     case 't':
       DisplayMode = DisplayCurses;
       break;
-    case 'a':
-      InterfaceAddress = optarg;
-      break;
     case 'g':
       DisplayMode = DisplayGTK;
       break;
-    case 's':                 /* BL */
+    case 'p':                 /* BL */
       DisplayMode = DisplaySplit;
       break;
     case 'l':
       DisplayMode = DisplayRaw;
       break;
+    case 'x':
+      DisplayMode = DisplayXML;
+      break;
+
+    case 'c':
+      MaxPing = atoi (optarg);
+      ForceMaxPing = 1;
+      break;
+    case 's':
+      packetsize = atoi (optarg);
+      if( packetsize >=0 ) {
+        if ( packetsize < MINPACKET ) packetsize = MINPACKET;
+        if ( packetsize > MAXPACKET ) packetsize = MAXPACKET;
+      }
+      break;
+    case 'a':
+      InterfaceAddress = optarg;
+      break;
     case 'n':
       dns = 0;
       break;
@@ -118,12 +168,54 @@ void parse_arg(int argc, char **argv) {
 	exit (1);
       }
       if (getuid() != 0 && WaitTime < 1.0)
-       WaitTime = 1.0;
+	WaitTime = 1.0;
+      break;
+    case 'f':
+      fstTTL = atoi (optarg);
+      if( fstTTL > maxTTL ) {
+	fstTTL = maxTTL;
+      }
+      if( fstTTL < 1) {                       /* prevent 0 hop */
+	fstTTL = 1;
+      }
+      break;
+    case 'm':
+      maxTTL = atoi (optarg);
+      if( maxTTL > (MaxHost - 1) ) {
+	maxTTL = MaxHost-1;
+      }
+      if( maxTTL < 1) {                       /* prevent 0 hop */
+	maxTTL = 1;
+      }
+      if( fstTTL > maxTTL ) {         /* don't know the pos of -m or -f */
+	fstTTL = maxTTL;
+      }
+      break;
+    case 'o':
+      /* XXX no error checking on the input string, lazy */
+      strncpy (fld_active, optarg, MAXFLD-1 );
+      break;
+    case 'b':
+      bitpattern = atoi (optarg);
+      if( bitpattern > 255 ) 
+	bitpattern = -1;
+      break;
+    case 'Q':
+      tos = atoi (optarg);
+      if( tos > 255 || tos <0 ) {
+	/* error message, should do more checking for valid values,
+	 * details in rfc2474 */
+	tos = 0;
+      }
       break;
     }
   }
-
-  if ((DisplayMode == DisplayReport) || (DisplayMode == DisplayRaw))
+  
+  if(DisplayMode == DisplayReport ||
+     DisplayMode == DisplayTXT ||
+     DisplayMode == DisplayXML ||
+     DisplayMode == DisplayRaw ||
+     DisplayMode == DisplayCSV )
     Interactive = 0;
 
   if(optind > argc - 1)
@@ -131,9 +223,13 @@ void parse_arg(int argc, char **argv) {
 
   Hostname = argv[optind++];
 
-  if (argc > optind) 
+  if (argc > optind) {
     packetsize = atoi(argv[optind]);
-
+    if( packetsize >=0 ) {
+      if ( packetsize < MINPACKET ) packetsize = MINPACKET;
+      if ( packetsize > MAXPACKET ) packetsize = MAXPACKET;
+    }
+  }
 }
 
 
@@ -161,9 +257,9 @@ void parse_mtr_options (char *string)
 
 
 int main(int argc, char **argv) {
-  int traddr;
-  struct hostent *host;
-  int net_preopen_result;
+  int               traddr;
+  struct hostent *  host                = NULL;
+  int               net_preopen_result;
 
   /*  Get the raw sockets first thing, so we can drop to user euid immediately  */
 
@@ -180,6 +276,9 @@ int main(int argc, char **argv) {
     printf("mtr: Unable to drop permissions.\n");
     exit(1);
   }
+
+  /* reset the random seed */
+  srand(getpid());
   
   display_detect(&argc, &argv);
 
@@ -212,7 +311,7 @@ int main(int argc, char **argv) {
   }
 
 
-  if(InterfaceAddress) { /* Mostly borrowed from ping(1) code */
+  if (InterfaceAddress) { /* Mostly borrowed from ping(1) code */
     int i1, i2, i3, i4;
     char dummy;
     extern int sendsock; /* from net.c:118 */
@@ -226,7 +325,7 @@ int main(int argc, char **argv) {
       printf("mtr: bad interface address: %s\n", InterfaceAddress);
       exit(1);
     } else {
-      unsigned char*ptr;
+      unsigned char *ptr;
       ptr = (unsigned char*)&sourceaddress.sin_addr;
       ptr[0] = i1;
       ptr[1] = i2;
@@ -239,14 +338,21 @@ int main(int argc, char **argv) {
       exit(1);
     }
   }
-  
-  host = gethostbyname(Hostname);
-  if(host == NULL) {
-#ifndef NO_HERROR
-    herror("mtr");
-#else
-    printf("mtr: error looking up \"%s\"\n", Hostname);
+
+#ifdef ENABLE_IPV6
+  if (af == AF_UNSPEC) {
+    af = AF_INET6;
+    host = gethostbyname2(Hostname, af);
+    if (host == NULL) af = AF_INET;
+  }
 #endif
+   
+  if (host == NULL) {
+    host = gethostbyname2(Hostname, af);
+  }
+  
+  if(host == NULL) {
+    herror("mtr");
     exit(1);
   }
 
@@ -257,6 +363,25 @@ int main(int argc, char **argv) {
     exit(1);
   }
 
+  switch (af) {
+    case AF_INET:
+      traddr = *(int *)host->h_addr;
+  
+      if(net_open(traddr) != 0) {
+        printf("mtr: Unable to get raw socket.  (Executable not suid?)\n");
+        exit(1);
+      }
+      break;
+#ifdef ENABLE_IPV6
+     case AF_INET6:
+       if(net6_open((struct in6_addr *)host->h_addr) != 0) {
+         printf("mtr: Unable to get raw socket.  (Executable not suid?)\n");
+         exit(1);
+       }
+       break;
+#endif
+  }
+
   display_open();
   dns_open();
 
